#define  STRICT
#include <windows.h>
#pragma hdrstop

#include <winioctl.h>
#include <windowsx.h>
#include <stdio.h>
#include <stdlib.h>

#include "flexdrv.h"
#include "drv32.h"
#include "pciscc.h"
#include "pciscc_ioctl.h"
#include "resource.h"

#define	DriverName	"PCISCCNT.SYS"
#define	DriverVer	"0.2c"
#define	ServiceName	"PCISCC"
#define	DeviceName0	"\\\\.\\PCISCC0"
#define	DeviceName1	"\\\\.\\PCISCC1"

/* global variables */
pciscc_state	state;
pciscc_config	config[4];

/* driver callback function list */
__declspec(dllexport) u32 flexnet_driver(int func, int chan, int arg1, int arg2)
{
	switch (func)
	{
		case drv_interfaceversion:
			return DRIVER_INTERFACE_VERSION;
		case drv_ident:
			return (u32)l1_ident((byte)(chan - state.base_chan));
		case drv_version:
			return (u32)l1_version((byte)(chan - state.base_chan));
		case drv_config:
			return config_device((byte)(chan), (HWND)arg1, (byte)(arg2 - state.base_chan));
		case drv_confinfo:
			return (u32)config_info((byte)(chan - state.base_chan));
		case drv_init_device:
			state.base_chan = chan;
			return init_device((HKEY)arg2);
		case drv_get_ch_cnt:
			return l1_get_ch_cnt();
		case drv_exit:
			l1_exit((HKEY)arg1);
			break;
		case drv_ch_active:
			return l1_ch_active((byte)(chan - state.base_chan));
		case drv_init_kanal:
			return l1_init_kanal((byte)(chan - state.base_chan), (u16)arg1, (u16)arg2);
		case drv_stat:
			return (u32)l1_stat((byte)(chan - state.base_chan), (byte)arg1);
		case drv_ch_state:
			return l1_ch_state((byte)(chan - state.base_chan));
		case drv_scale:
			return config[chan - state.base_chan].scale;
		case drv_tx_calib:
			l1_tx_calib((byte)(chan - state.base_chan), (byte)arg1);
			break;
		case drv_set_led:
			break;
		case drv_rx_frame:
			return (u32)l1_rx_frame();
		case drv_get_framebuf:
			return (u32)l1_get_framebuf((byte)(chan - state.base_chan));
		case drv_tx_frame:
			return l1_tx_frame();
		case drv_get_txdelay:
			return config[chan - state.base_chan].txdelay / 10;
		case drv_get_mode:
			return config[chan - state.base_chan].mode;
		case drv_get_baud:
			return config[chan - state.base_chan].baud / 100;
		case drv_set_txdelay:
			config[chan - state.base_chan].txdelay = arg1 * 10;
			break;
		default:
			return 0;
	}
	return 0;
}

BOOL WINAPI DllMain (HANDLE hModule, DWORD dwFunction, LPVOID lpNot)
{
	switch(dwFunction)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
			state.hInst = hModule;
			break;
		case DLL_PROCESS_DETACH:
		case DLL_THREAD_DETACH:
			break;
	}
	return 1;
}

static BOOL CALLBACK EdParmDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	char buf[32];
	int i,idx;
	int value;

	switch (uMsg)
	{
		case WM_INITDIALOG:
			idx = 0;
			for (i = 1200; i <= 1228800; i *= 2)
			{
				sprintf(buf,"%d",i);
				SendMessage(GetDlgItem(hDlg,IDC_BAUDRATE_0),CB_ADDSTRING,0,(LPARAM)buf);
				SendMessage(GetDlgItem(hDlg,IDC_BAUDRATE_1),CB_ADDSTRING,0,(LPARAM)buf);
				SendMessage(GetDlgItem(hDlg,IDC_BAUDRATE_2),CB_ADDSTRING,0,(LPARAM)buf);
				SendMessage(GetDlgItem(hDlg,IDC_BAUDRATE_3),CB_ADDSTRING,0,(LPARAM)buf);
				if (config[0].baud == i)
					ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_0),idx);
				if (config[1].baud == i)
					ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_1),idx);
				if (config[2].baud == i)
					ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_2),idx);
				if (config[3].baud == i)
					ComboBox_SetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_3),idx);
				idx++;
			}

			sprintf(buf,"%d",config[0].txdelay);
			SetDlgItemText(hDlg,IDC_TXDELAY_0,buf);
			sprintf(buf, "%d", config[1].txdelay);
			SetDlgItemText(hDlg,IDC_TXDELAY_1,buf);
			sprintf(buf,"%d",config[2].txdelay);
			SetDlgItemText(hDlg,IDC_TXDELAY_2,buf);
			sprintf(buf,"%d",config[3].txdelay);
			SetDlgItemText(hDlg,IDC_TXDELAY_3,buf);

			CheckDlgButton(hDlg,IDC_DUPLEX_0,config[0].mode & MODE_d ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_DUPLEX_1,config[1].mode & MODE_d ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_DUPLEX_2,config[2].mode & MODE_d ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_DUPLEX_3,config[3].mode & MODE_d ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_RXCLOCK_0,config[0].mode & MODE_r ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_RXCLOCK_1,config[1].mode & MODE_r ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_RXCLOCK_2,config[2].mode & MODE_r ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_RXCLOCK_3,config[3].mode & MODE_r ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_TXCLOCK_0,config[0].mode & MODE_t ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_TXCLOCK_1,config[1].mode & MODE_t ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_TXCLOCK_2,config[2].mode & MODE_t ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_TXCLOCK_3,config[3].mode & MODE_t ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_NRZCODE_0,config[0].mode & MODE_z ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_NRZCODE_1,config[1].mode & MODE_z ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_NRZCODE_2,config[2].mode & MODE_z ? BST_CHECKED : BST_UNCHECKED);
			CheckDlgButton(hDlg,IDC_NRZCODE_3,config[3].mode & MODE_z ? BST_CHECKED : BST_UNCHECKED);
			break;
                
        case WM_COMMAND:
			switch (GET_WM_COMMAND_ID(wParam, lParam))
			{
				case IDC_TXD_UP_0:
					GetDlgItemText(hDlg,IDC_TXDELAY_0,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value < 2550)
						value += 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_0,buf);
					break;

				case IDC_TXD_DOWN_0:
					GetDlgItemText(hDlg,IDC_TXDELAY_0,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value >= 10)
						value -= 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_0,buf);
					break;

				case IDC_TXD_UP_1:
					GetDlgItemText(hDlg,IDC_TXDELAY_1,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value < 2550)
						value += 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_1,buf);
					break;

				case IDC_TXD_DOWN_1:
					GetDlgItemText(hDlg,IDC_TXDELAY_1,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value >= 10)
						value -= 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_1,buf);
					break;

				case IDC_TXD_UP_2:
					GetDlgItemText(hDlg,IDC_TXDELAY_2,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value < 2550)
						value += 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_2,buf);
					break;

				case IDC_TXD_DOWN_2:
					GetDlgItemText(hDlg,IDC_TXDELAY_2,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value >= 10)
						value -= 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_2,buf);
					break;

				case IDC_TXD_UP_3:
					GetDlgItemText(hDlg,IDC_TXDELAY_3,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value < 2550)
						value += 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_3,buf);
					break;

				case IDC_TXD_DOWN_3:
					GetDlgItemText(hDlg,IDC_TXDELAY_3,buf,sizeof(buf));
					value = strtol(buf,NULL,0);
					if (value >= 10)
						value -= 10;
					sprintf(buf,"%d",value);
					SetDlgItemText(hDlg,IDC_TXDELAY_3,buf);
					break;

				case IDCANCEL:
					EndDialog(hDlg, 0);
					break;

				case IDOK:
					config[0].mode = 0;
					config[0].mode |= (IsDlgButtonChecked(hDlg, IDC_DUPLEX_0) == BST_CHECKED) ? MODE_d : 0;
					config[0].mode |= (IsDlgButtonChecked(hDlg, IDC_NRZCODE_0) == BST_CHECKED) ? MODE_z : 0;
					config[0].mode |= (IsDlgButtonChecked(hDlg, IDC_RXCLOCK_0) == BST_CHECKED) ? MODE_r : 0;
					config[0].mode |= (IsDlgButtonChecked(hDlg, IDC_TXCLOCK_0) == BST_CHECKED) ? MODE_t : 0;
					config[1].mode = 0;
					config[1].mode |= (IsDlgButtonChecked(hDlg, IDC_DUPLEX_1) == BST_CHECKED) ? MODE_d : 0;
					config[1].mode |= (IsDlgButtonChecked(hDlg, IDC_NRZCODE_1) == BST_CHECKED) ? MODE_z : 0;
					config[1].mode |= (IsDlgButtonChecked(hDlg, IDC_RXCLOCK_1) == BST_CHECKED) ? MODE_r : 0;
					config[1].mode |= (IsDlgButtonChecked(hDlg, IDC_TXCLOCK_1) == BST_CHECKED) ? MODE_t : 0;
					config[2].mode = 0;
					config[2].mode |= (IsDlgButtonChecked(hDlg, IDC_DUPLEX_2) == BST_CHECKED) ? MODE_d : 0;
					config[2].mode |= (IsDlgButtonChecked(hDlg, IDC_NRZCODE_2) == BST_CHECKED) ? MODE_z : 0;
					config[2].mode |= (IsDlgButtonChecked(hDlg, IDC_RXCLOCK_2) == BST_CHECKED) ? MODE_r : 0;
					config[2].mode |= (IsDlgButtonChecked(hDlg, IDC_TXCLOCK_2) == BST_CHECKED) ? MODE_t : 0;
					config[3].mode = 0;
					config[3].mode |= (IsDlgButtonChecked(hDlg, IDC_DUPLEX_3) == BST_CHECKED) ? MODE_d : 0;
					config[3].mode |= (IsDlgButtonChecked(hDlg, IDC_NRZCODE_3) == BST_CHECKED) ? MODE_z : 0;
					config[3].mode |= (IsDlgButtonChecked(hDlg, IDC_RXCLOCK_3) == BST_CHECKED) ? MODE_r : 0;
					config[3].mode |= (IsDlgButtonChecked(hDlg, IDC_TXCLOCK_3) == BST_CHECKED) ? MODE_t : 0;

					GetDlgItemText(hDlg,IDC_TXDELAY_0,buf,sizeof(buf));
					config[0].txdelay = strtol(buf,NULL,0);
					GetDlgItemText(hDlg,IDC_TXDELAY_1,buf,sizeof(buf));
					config[1].txdelay = strtol(buf,NULL,0);
					GetDlgItemText(hDlg,IDC_TXDELAY_2,buf,sizeof(buf));
					config[2].txdelay = strtol(buf,NULL,0);
					GetDlgItemText(hDlg,IDC_TXDELAY_3,buf,sizeof(buf));
					config[3].txdelay = strtol(buf,NULL,0);

					config[0].baud = (1 << ComboBox_GetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_0))) * 1200;
					config[1].baud = (1 << ComboBox_GetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_1))) * 1200;
					config[2].baud = (1 << ComboBox_GetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_2))) * 1200;
					config[3].baud = (1 << ComboBox_GetCurSel(GetDlgItem(hDlg,IDC_BAUDRATE_3))) * 1200;

					EndDialog(hDlg, 1);
					break;
			}
			break;
	
		default:
			return FALSE;
	}
	return TRUE;
}

int system_isnt4(void)
{
	OSVERSIONINFO OsVer;

	OsVer.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx((OSVERSIONINFO *)&OsVer))
		return 0;
	if (OsVer.dwPlatformId != VER_PLATFORM_WIN32_NT)
		return 0;
	if (OsVer.dwMajorVersion != 4)
		return 0;
	return 1;
}

int flex_home(char *Path, int Len)
{
	HKEY	hKeyFlex;
	long	Status;

	Status = RegOpenKeyEx(	HKEY_LOCAL_MACHINE,TEXT("SOFTWARE\\FlexNet"),0L,
							KEY_READ,&hKeyFlex);
	if (Status != ERROR_SUCCESS)
		return 0;
	Status = RegQueryValueEx(hKeyFlex,TEXT("Home"),NULL,NULL,Path,&Len);
	RegCloseKey(hKeyFlex);
	if (Status != ERROR_SUCCESS)
		return 0;
	return strlen(Path);
}

int create_service(char *Name, char *Path)
{
	SC_HANDLE		scm;
	SC_HANDLE		scs;
	int				ret = 0;

	scm = OpenSCManager(NULL,NULL,SC_MANAGER_ALL_ACCESS);
	if (!scm)
		return 0;
	scs = CreateService(scm,Name,Name,SERVICE_ALL_ACCESS,
						SERVICE_KERNEL_DRIVER,SERVICE_DEMAND_START,
						SERVICE_ERROR_NORMAL,Path,NULL,NULL,NULL,NULL,NULL);
	if (scs == NULL)
		scs = OpenService(scm,Name,SERVICE_ALL_ACCESS);
	if (scs != NULL)
	{ 
		StartService(scs,0,NULL);
		CloseServiceHandle(scs);
		ret = 1;
	}
	CloseServiceHandle(scm);
	return ret;
}

int remove_service(char *Name)
{
	SC_HANDLE		scm;
	SC_HANDLE		scs;
	SERVICE_STATUS	ss;
	int				ret = 0;

	scm = OpenSCManager(NULL,NULL,SC_MANAGER_ALL_ACCESS);
	if (!scm)
		return 0;
	scs = OpenService(scm,Name,SERVICE_ALL_ACCESS);
	if (scs != NULL)
	{
		ControlService(scs,SERVICE_CONTROL_STOP,&ss);
		DeleteService(scs);
		CloseServiceHandle(scs);
		ret = 1;
	}
	CloseServiceHandle(scm);
	return ret;
}

void read_config(HKEY hKey)
{
	DWORD	reglen;

	config[0].baud = 1200;
	config[0].mode = 0;
	config[0].txdelay = 200;

	reglen = sizeof(config[0].baud);
	RegQueryValueEx(hKey,"Baud0",NULL,NULL,(void*)&config[0].baud,&reglen);
	reglen = sizeof(config[0].mode);
	RegQueryValueEx(hKey,"Mode0",NULL,NULL,(void*)&config[0].mode,&reglen);
	reglen = sizeof(config[0].txdelay);
	RegQueryValueEx(hKey,"TxDelay0",NULL,NULL,(void*)&config[0].txdelay,&reglen);

	config[1].baud = 1200;
	config[1].mode = 0;
	config[1].txdelay = 200;

	reglen = sizeof(config[1].baud);
	RegQueryValueEx(hKey,"Baud1",NULL,NULL,(void*)&config[1].baud,&reglen);
	reglen = sizeof(config[1].mode);
	RegQueryValueEx(hKey,"Mode1",NULL,NULL,(void*)&config[1].mode,&reglen);
	reglen = sizeof(config[1].txdelay);
	RegQueryValueEx(hKey,"TxDelay1",NULL,NULL,(void*)&config[1].txdelay,&reglen);

	config[2].baud = 1200;
	config[2].mode = 0;
	config[2].txdelay = 200;

	reglen = sizeof(config[2].baud);
	RegQueryValueEx(hKey,"Baud2",NULL,NULL,(void*)&config[2].baud,&reglen);
	reglen = sizeof(config[2].mode);
	RegQueryValueEx(hKey,"Mode2",NULL,NULL,(void*)&config[2].mode,&reglen);
	reglen = sizeof(config[2].txdelay);
	RegQueryValueEx(hKey,"TxDelay2",NULL,NULL,(void*)&config[2].txdelay,&reglen);

	config[3].baud = 1200;
	config[3].mode = 0;
	config[3].txdelay = 200;

	reglen = sizeof(config[3].baud);
	RegQueryValueEx(hKey,"Baud3",NULL,NULL,(void*)&config[3].baud,&reglen);
	reglen = sizeof(config[3].mode);
	RegQueryValueEx(hKey,"Mode3",NULL,NULL,(void*)&config[3].mode,&reglen);
	reglen = sizeof(config[3].txdelay);
	RegQueryValueEx(hKey,"TxDelay3",NULL,NULL,(void*)&config[3].txdelay,&reglen);
}

void write_config(HKEY hKey)
{
	RegSetValueEx(hKey,"Baud0",0,REG_DWORD,(void*)&config[0].baud,sizeof(config[0].baud));
	RegSetValueEx(hKey,"Mode0",0,REG_DWORD,(void*)&config[0].mode,sizeof(config[0].mode));
	RegSetValueEx(hKey,"TxDelay0",0,REG_DWORD,(void*)&config[0].txdelay,sizeof(config[0].txdelay));

	RegSetValueEx(hKey,"Baud1",0,REG_DWORD,(void*)&config[1].baud,sizeof(config[1].baud));
	RegSetValueEx(hKey,"Mode1",0,REG_DWORD,(void*)&config[1].mode,sizeof(config[1].mode));
	RegSetValueEx(hKey,"TxDelay1",0,REG_DWORD,(void*)&config[1].txdelay,sizeof(config[1].txdelay));

	RegSetValueEx(hKey,"Baud2",0,REG_DWORD,(void*)&config[2].baud,sizeof(config[2].baud));
	RegSetValueEx(hKey,"Mode2",0,REG_DWORD,(void*)&config[2].mode,sizeof(config[2].mode));
	RegSetValueEx(hKey,"TxDelay2",0,REG_DWORD,(void*)&config[2].txdelay,sizeof(config[2].txdelay));

	RegSetValueEx(hKey,"Baud3",0,REG_DWORD,(void*)&config[3].baud,sizeof(config[3].baud));
	RegSetValueEx(hKey,"Mode3",0,REG_DWORD,(void*)&config[3].mode,sizeof(config[3].mode));
	RegSetValueEx(hKey,"TxDelay3",0,REG_DWORD,(void*)&config[3].txdelay,sizeof(config[3].txdelay));
}

int check_driver(int num)
{
	HANDLE hndTest = INVALID_HANDLE_VALUE;
	long result = 0;
	long open = 0;

	if (num == 0)
		hndTest = CreateFile(	DeviceName0,GENERIC_READ | GENERIC_WRITE,
								0,NULL,OPEN_EXISTING,0,NULL);
	if (num == 1)
		hndTest = CreateFile(	DeviceName1,GENERIC_READ | GENERIC_WRITE,
								0,NULL,OPEN_EXISTING,0,NULL);
	if (hndTest == INVALID_HANDLE_VALUE)
		return 0;

	DeviceIoControl(hndTest,PCISCC_GET_OPEN,NULL,0,&open,sizeof(ULONG),&result,NULL);
	CloseHandle(hndTest);

	return (open == 1);
}

int init_device(HKEY hKey)
{
	long result;

	if (system_isnt4())
	{
		char DriverPath[256];
		if (!flex_home(DriverPath,200))
			return 0;
		strcat(DriverPath,DriverName);
		create_service(ServiceName,DriverPath);
	}
	state.restart = 0;
	state.card = -1;

	if (check_driver(0))
		state.card = 0;
	else if (check_driver(1))
		state.card = 1;

	switch (state.card) {
	case 0:
		state.hndCtrl = CreateFile(	DeviceName0,GENERIC_READ | GENERIC_WRITE,
									0,NULL,OPEN_EXISTING,0,NULL);
		state.hndRead = CreateFile(	DeviceName0,GENERIC_READ | GENERIC_WRITE,
									0,NULL,OPEN_EXISTING,0,NULL);
		break;
	case 1:
		state.hndCtrl = CreateFile(	DeviceName1,GENERIC_READ | GENERIC_WRITE,
									0,NULL,OPEN_EXISTING,0,NULL);
		state.hndRead = CreateFile(	DeviceName1,GENERIC_READ | GENERIC_WRITE,
									0,NULL,OPEN_EXISTING,0,NULL);
		break;
	default:
		return 0;
	}

	DeviceIoControl(state.hndCtrl,PCISCC_BLOCK_READ,NULL,0,NULL,0,&result,NULL);
	read_config(hKey);

	return 1;
}

void l1_exit(HKEY hKey)
{
	long result;

	DeviceIoControl(state.hndCtrl,PCISCC_NONBLOCK_READ,NULL,0,NULL,0,&result,NULL);
	CloseHandle(state.hndCtrl);
	CloseHandle(state.hndRead);
	state.hndCtrl = INVALID_HANDLE_VALUE;
	state.hndRead = INVALID_HANDLE_VALUE;

	write_config(hKey);

	if (state.restart)
		return;
	if (system_isnt4())
		remove_service(ServiceName);
}

int config_device(byte max_channels, HWND hDlg, byte channel)
{
	if (state.restart = DialogBoxParam(state.hInst,MAKEINTRESOURCE(IDD_DIALOG1),hDlg,EdParmDlgProc,0))
		return 4;
	return 0;
}

byte *config_info(byte chan)
{
	static char buf[32];

	sprintf(buf,"Channel %d",chan);
	return buf;
}

byte l1_init_kanal(byte chan, u16 baud, u16 mode)
{
	long arg[2];
	long bytes = 0;
	long cmd;

	arg[0] = config[chan].baud = baud * 100;
	arg[1] = config[chan].mode = mode;

	if (config[chan].baud)
		config[chan].scale = 614400 / config[chan].baud;

	switch (chan)
	{
		case 0: cmd = PCISCC_SET_PARAM_0;
				break;
		case 1: cmd = PCISCC_SET_PARAM_1;
				break;
		case 2: cmd = PCISCC_SET_PARAM_2;
				break;
		case 3: cmd = PCISCC_SET_PARAM_3;
				break;
	}
	DeviceIoControl(state.hndCtrl,cmd,&arg,sizeof(arg),
					NULL,0,&bytes,NULL);
	if (!bytes)
		return 0;
	return 1;
}

byte l1_ch_state(byte chan)
{
	long arg = 0xff;
	long bytes;
	long cmd;

	switch (chan)
	{
		case 0: cmd = PCISCC_GET_STATE_0;
				break;
		case 1: cmd = PCISCC_GET_STATE_1;
				break;
		case 2: cmd = PCISCC_GET_STATE_2;
				break;
		case 3: cmd = PCISCC_GET_STATE_3;
				break;
	}
	DeviceIoControl(state.hndCtrl,cmd,NULL,0,&arg,sizeof(arg),
					&bytes,NULL);
	return (byte)arg;
}

void l1_tx_calib(byte chan, byte min)
{
	long arg = min;
	long bytes;
	long cmd;

	switch (chan)
	{
		case 0: cmd = PCISCC_SET_CALIB_0;
				break;
		case 1: cmd = PCISCC_SET_CALIB_1;
				break;
		case 2: cmd = PCISCC_SET_CALIB_2;
				break;
		case 3: cmd = PCISCC_SET_CALIB_3;
				break;
	}
	DeviceIoControl(state.hndCtrl,cmd,&arg,sizeof(arg),NULL,0,
					&bytes,NULL);
}

L1_STATISTICS *l1_stat(byte chan, byte del)
{
	static L1_STATISTICS info;
	long bytes;
	long cmd;

	switch (chan)
	{
		case 0: cmd = PCISCC_GET_INFO_0;
				break;
		case 1: cmd = PCISCC_GET_INFO_1;
				break;
		case 2: cmd = PCISCC_GET_INFO_2;
				break;
		case 3: cmd = PCISCC_GET_INFO_3;
				break;
	}
	memset(&info,0,sizeof(L1_STATISTICS));
	DeviceIoControl(state.hndCtrl,cmd,NULL,0,&info,sizeof(L1_STATISTICS),
					&bytes,NULL);
	return &info;
}

L1FRAME *l1_rx_frame()
{
	long bytes = 0;

	while (1) {
		if (state.hndRead == INVALID_HANDLE_VALUE)
			return NULL;
		if (!ReadFile(state.hndRead,&state.rx_frame,sizeof(state.rx_frame),&bytes,NULL))
			return NULL;
		if (bytes)
			break;
	}
	state.rx_frame.kanal += state.base_chan;
	return &state.rx_frame;
}

byte l1_tx_frame(void)
{
	long bytes = 0;

	state.tx_frame.kanal = state.tx_chan;
	state.tx_frame.txdelay = config[state.tx_chan].txdelay / 10;

	WriteFile(state.hndCtrl,&state.tx_frame,sizeof(state.tx_frame),&bytes,NULL);
	if (!bytes)
		return 0;
	return 1;
}

L1FRAME *l1_get_framebuf(byte chan)
{
	state.tx_chan = chan;
	return &state.tx_frame;
}

char *l1_ident(byte chan)
{
	switch (state.card) {
		case 0:	return "PCISCC (0)";
		case 1:	return "PCISCC (1)";
		default: return "PCISCC";
	}
}

char *far l1_version(byte chan)
{
	return DriverVer;
}

u16 l1_get_ch_cnt(void)
{
	return 4;
}

byte l1_ch_active(byte chan)
{
	return 1;
}
